#!/usr/local/bin/php -q -n
<?php
//This script must be in drupal/sites/all/modules/mail_import -directory.
$to_root = '../../../../';
require_once '../../../default/settings.php';

if(!isset($db_url)){
	die('Invalid settings file, db_url not found.');
}
$url = parse_url($db_url);

$dbhost = $url['host'];
$dbuser = $url['user'];;
$dbname = $url['path'];
$dbpass = $url['pass'];

if($dbhost == null || $dbuser == null || $dbname == null || $dbpass == null){
	die('Some database logins missing.');
}
$dbname = substr($dbname, 1);

require_once('rfc822_addresses.php');
require_once('mime_parser.php');
require_once("class.inputfilter.php");
require_once("db_mysql.php");

if(!log_open('log.txt')){
	die('Could not open log file.');
}

$email_file = fopen("php://stdin", "r");
$str = '';
while($line = fgets($email_file)){
	$str .= $line;
}
fclose($email_file);


$mime=new mime_parser_class;
$mime->mbox = 1;
$mime->decode_bodies = 1;
$mime->ignore_syntax_errors = 1;
$parameters=array(
	'Data'=> $str,           
);
if(!$mime->Decode($parameters, $decoded)){
	$err = 'MIME message decoding error: '.$mime->error.' at position '.$mime->error_position."\n";
	log_log($err);
	die($err);
}

if(!db_connect($dbhost, $dbuser, $dbpass, $dbname)){
	$err ='Error connecting to mysql, see log file.';
	log_log($err);
	die($err);
}

$result = db_query('select * from mail_import_settings order by id');
if($result == null){
	$err = 'Could not get filtering rules from database';
	log_log($err);
	die($err);
}

$counter = 0;
while($res = db_fetch_object($result)){
	$rules[$counter] = array(
		'id' => $res->id,
		'cond_str'=> $res->cond_str,
		'site'=> $res->site,
		'title_field'=> $res->title_field,
		'check_action'=> $res->check_action,
		'table_prefix'=> optionStrToArray($res->table_prefix),
		'node_type'=> $res->node_type,
		'tid'=> $res->tid,
		'lang'=> $res->lang,
		'from_field'=> $res->from_field,		
		'default_owner'=> $res->default_owner,
		'options'=> $res->options,
		'attachments_dir'=> $res->attachments_dir,
		'access_rights' => $res->access_rights
		);
	$counter++;
}
if($counter == 0){
	$err = 'No filtering rules found from database, please add it using mail import GUI in your drupal site.';
	log_log($err);
	die($err);	
}

//Set the global variable
$sequences_table_name = $rules[0]['table_prefix']['sequences']. 'sequences';

$inputFilter = new InputFilter();

//var_dump($decoded);
//die('test');
for($message = 0; $message < count($decoded); $message++){
	processMessage($decoded[$message], $inputFilter, $rules);
}  
db_query('commit');
db_close();
log_close();


//======================================================================================== 
function optionStrToArray($str){
	$opts = explode(',', $str);
	$ret;
	foreach($opts as $opt){
		$value = explode('=', $opt);
		$ret[$value[0]] = $value[1];	
	}	
	return $ret;	
}

function convertUTF8($str){
	if(strpos(strtolower($str), 'utf') > 0){
		echo 'moikka';
		return utf8_decode($str);
	} else {
		return $str;
	}		
}

function processMessage($msg, $inputFilter, $rules){
	$node = new stdClass();
	$node->title = $inputFilter->process($msg["DecodedHeaders"]["subject:"][0][0]["Value"]);
	if($node->title == null){
		$node->title = $inputFilter->process($msg["Headers"]["subject:"]);
	}
	
	
	$from = $msg["DecodedHeaders"]["from:"][0][0]["Value"]; //$inputFilter->process($msg["DecodedHeaders"]["from:"][0][0]["Value"]);
	if($msg["DecodedHeaders"]["from:"][0][0]['Encoding'] == 'UTF-8'){
		$from = utf8_decode($from);
	};
	if($from == null){
		$from = convertUTF8($msg["Headers"]["from:"]); //$inputFilter->process($msg["Headers"]["from:"]);
	}
	
	$to = $inputFilter->process($msg["DecodedHeaders"]["to:"][0][0]["Value"]);
	if($msg["DecodedHeaders"]["to:"][0][0]["Encoding"] == 'UTF-8'){
		$to = utf8_decode($to);
	}
	if($to == null){
		$to = convertUTF8($inputFilter->process($msg["Headers"]["to:"]));
	}
	
	$cc = $inputFilter->process($msg["DecodedHeaders"]["cc:"][0][0]["Value"]);
	if($msg["DecodedHeaders"]["cc:"][0][0]["Encoding"] == 'UTF-8'){
		$cc = utf8_decode($cc);
	}
	if($cc == null){
		$cc = convertUTF8($inputFilter->process($msg["Headers"]["cc:"]));
	}
		
	$date = $msg["Headers"]["date:"]; 
	if (($timestamp = strtotime($date)) === false) {
		$node->created = now();
		//echo ("Could not parse date");
	} else {
		$node->created = date('U', $timestamp);
		//echo('###DATE:' . $node->created);
	}
	
	$node->name = '';
	$node->from = $from;
	$node->status = 1;	
	$node->log = null;
	$node->comment = null;
	$node->promote =0;
	$node->sticky=0;
	$node->format = null;		
	
	foreach($rules as $rule){
		$fields = null;
		$match_value = $rule['cond_str'];
		switch($rule['title_field']){
			case 0:
				$fields = array($node->title);
				break;

			/*
			case 0:
				$fields = array($to);
				break;
			case 1:
				$fields = array($cc);
				break;
			case 2:
				$fields = array($cc, $to);
				break;
			case 3:
				$fields = array($from);
				break;			
			case 4:
				$fields = array($node->title);
				break;
			*/
			default: 
				$fields = array();
				break;
		}
		
		$matches_rule = false;		
		switch($rule['check_action']){
			case 0:				
				foreach($fields as $field){
					if(strpos($field, $match_value) !== false){
						$matches_rule = true;
					}
				}
				break;
			case 1:
				foreach($fields as $field){
					if($field == $match_value){
						$matches_rule = true;
					}
				}
				break;
			case 2:
				foreach($fields as $field){
					if(strpos($field, $match_value) === 0){
						$matches_rule = true;
					}
				}
				break;
		}
		if(!$matches_rule){
			/*
			$file = fopen('filtered_mail.txt', 'a') or die('Cannot open file');
			fwrite($file, "Did not match any criteria:\n");
			fwrite($file, $node->title . "\n");
			fwrite($file, $from . "\n");
			fwrite($file, $to . "\n");
			fwrite($file, "-----------------------\n \n \n");
			fclose($file);
			*/			
			continue;
		}
		
		if(!isset($node->body)){
			if(isset($msg["Body"])){
				$body = $inputFilter->process($msg["Body"]);
			} else {
				$body = '';
			}
			if(isset($msg['Headers']['content-type:']) && strpos(strtolower($msg['Headers']['content-type:']), 'utf')){
				$body = utf8_decode($body);
			}		
			$node->body = $body;
		}
		
		$node->type = $rule['node_type'];
		$node->uid = $rule['default_owner'];		
		//Save possible attachments
		$files = null;
		$files_dir = $rule['attachments_dir'];
		if(substr($files_dir, strlen($files_dir)-1,1) != '/'){
			$files_dir .='/';
		}
		global $to_root;
		
		if(isset($msg['Parts']) &&  $files_dir != null && trim($files_dir) != ''){
			$i=0;	
			foreach($msg['Parts'] as $part){
				 $file = null;
				 $file = saveAttachmentPart($to_root . $files_dir, $part, $node, $i, $inputFilter);
				 if($file != null){
					 $files[] = $file;
				 }
				 $i++;
			}
		}
		
		
		if(!isset($node->teaser)){
				//Some mime messages have the body text as plain text attachment and was inserted to body in saveAttachmentPart
				//$node->body .= '<p class="email-address">' . $from . '</p>';
				$node->teaser = substr($node->body, 0, 200);
		}
		
		
		$node = node_save($node, $rule);	
		$nid = $node->nid;
		$vid = $node->vid;
		if($files != null){
			$files_table = $rule['table_prefix']['files'] . 'files';
			$files_fid_seq = $rule['table_prefix']['files'] . 'files_fid';
			$files_revisions_table = $rule['table_prefix']['file_revisions'] . 'file_revisions';			
			foreach($files as $f){
				$fid = db_next_id($files_fid_seq);
				$size = $f['size'];
				if($size == null){
					$size = 0;
				}
				$filename =  $f['orig_filename'];
				$path =  substr($f['path'], strlen($to_root));
				db_query("insert into $files_table (fid,nid,filename,filepath,filemime,filesize) values ($fid,$nid,".
					 "'". $filename ."','" . $path ."','',". $size .")" );
				db_query("insert into $files_revisions_table (fid,vid,description, list) values ($fid, $vid,'$filename', 1)");		
			}
		}
		
		
	}
}

function saveAttachmentPart($files_dir, $part, $node, $attch_nmb, $inputFilter){
	if(!isset($part['Headers']['content-type:'])){
		return;
	}
	$content_type = strtolower($part['Headers']['content-type:']);
	if(strpos($content_type, 'plain') > 0){
	   //Save plain text to the node body
	   $new_body = $part['Body'];
	   if(strpos($content_type,'utf-8') || strpos($content_type,'utf8')){
		   $new_body = utf8_decode($new_body);
	   }		   	   
	   $new_body = $inputFilter->process($new_body);
	   if(strpos($node->body, $new_body) === false){
		   $node->body .= $new_body;
	   }	  
	  return;
	}
	if(!isset($part['FileName'])){
		return;
	} 
 	$filename=trim($part['FileName']);	
	if(!isset($part['Body'])){
		return;
	}
 	$body=$part['Body'];	
	if($filename == '-' || $filename == ''){
		if(strpos(strtolower($body), 'html')){
			$filename = $attch_nmb .'.html';
		} else {
			return;
		}
	}

	$date = strtotime($node->created);	
	$timestamp = gmdate('Y', $date) .'-'. two_digit(gmdate('m', $date)) .'-'. two_digit(gmdate('d', $date)) .'_' . two_digit(gmdate('H', $date)) . two_digit(gmdate('i', $date));		
	$orig_filename = $filename;
	$filename = $timestamp .'_' .replace_illegal_filename_chars($node->title) . '_' . replace_illegal_filename_chars($filename,false);
	$filepath = $files_dir . $filename;
	$file = fopen($filepath, 'w');
	fwrite($file, $body);
	fclose($file);
	return array('orig_filename'=>$orig_filename, 'filename'=>$filename, 'path'=>$filepath, 'size'=>filesize($filepath));	
}
function replace_illegal_filename_chars($str, $replace_dots = true){
	$str = strtolower($str);
	$str = str_replace(' ', '_', $str);
	if($replace_dots){
		$str = preg_replace('/[^0-9a-z_]+/i', '', $str);
	} else {
		$str = preg_replace('/[^0-9a-z_\.]+/i', '', $str);		
	}
	return $str;

}


/**
If number contains only 1 digit, inserts 0 before. So eg. 1 -> 01, 9->09, 12->12.
*/
function two_digit($number){
	if(strlen($number) == 1){
		return '0'. $number;
	} else {
		return $number;
	}
}

/**
 * Save a node object into the database (trimmed version from drupal node class)
 */
function node_save(&$node, $rule) {
    $node_table = $rule['table_prefix']['node'] .'node';
    $node_revisions_table = $rule['table_prefix']['node_revisions'] .'node_revisions';
    $comment_table = $rule['table_prefix']['node_comment_statistics'] . 'node_comment_statistics';
    $term_table = $rule['table_prefix']['term_node'] . 'term_node';
    $node_access_table = $rule['table_prefix']['node'].'node_access';
    $cache_table_page = $rule['table_prefix']['cache_page'].'cache_page';
    if(isset($rule['table_prefix']['cache_views'])){
	    $cache_table_views = $rule['table_prefix']['cache_views'].'cache_views';
    } else {
	    $cache_table_views = null;
    }
    
    
    
    $node->is_new = TRUE;
    $node->nid = db_next_id($node_table .'_nid');
    $node->vid = db_next_id($node_revisions_table .'_vid');
    //$node->created = time();
    $node->changed = time();

  // Split off revisions data to another structure
  $revisions_table_values = array('nid' => $node->nid, 'vid' => $node->vid,
                     'title' => $node->title, 'body' => $node->body,
                     'teaser' => $node->teaser, 'timestamp' => $node->changed,
                     'uid' => $node->uid, 'format' => $node->format);
  $revisions_table_types = array('nid' => '%d', 'vid' => '%d',
                     'title' => "'%s'", 'body' => "'%s'",
                     'teaser' => "'%s'", 'timestamp' => '%d',
                     'uid' => '%d', 'format' => '%d');
  if (!empty($node->log) || $node->is_new || $node->revision) {
    // Only store the log message if there's something to store; this prevents
    // existing log messages from being unintentionally overwritten by a blank
    // message. A new revision will have an empty log message (or $node->log).
    $revisions_table_values['log'] = $node->log;
    $revisions_table_types['log'] = "'%s'";
  }
  $node_table_values = array('nid' => $node->nid, 'vid' => $node->vid,
                    'title' => $node->title, 'type' => $node->type, 'uid' => $node->uid,
                    'status' => $node->status, 'created' => $node->created,
                    'changed' => $node->changed, 'comment' => $node->comment,
                    'promote' => $node->promote, 'sticky' => $node->sticky);
  $node_table_types = array('nid' => '%d', 'vid' => '%d',
                    'title' => "'%s'", 'type' => "'%s'", 'uid' => '%d',
                    'status' => '%d', 'created' => '%d',
                    'changed' => '%d', 'comment' => '%d',
                    'promote' => '%d', 'sticky' => '%d');

  //Generate the node table query and the
  //the node_revisions table query
  $node_query = "INSERT INTO $node_table (". implode(', ', array_keys($node_table_types)) .') VALUES ('. implode(', ', $node_table_types) .')';
  $revisions_query = "INSERT INTO $node_revisions_table (". implode(', ', array_keys($revisions_table_types)) .') VALUES ('. implode(', ', $revisions_table_types) .')'; 
  
  db_query($node_query, $node_table_values); 
  db_query($revisions_query, $revisions_table_values);

  //Make node to be show up on tracker -module
  $sql = 'insert into ' . $comment_table .' (nid, last_comment_timestamp, last_comment_uid, comment_count) values ( '. $node->nid . ','. $node->changed . ',' . $node->uid . ',0)' ;
  db_query($sql);
  
  //Add term assignment  
  if($rule['tid'] > -1){
	  $sql = 'insert into '  . $term_table . ' (nid,tid) values (' . $node->nid . ','. $rule['tid']. ')' ;
	  db_query($sql);
  }
  
  //Add sender address to cck field if configured so
  if($rule['from_field'] != null){
	  $table = $rule['table_prefix']['content_type_table'] . 'content_type_' . $node->type;
	  $column = $rule['from_field'] . '_value';
	  $sql = 'insert into '  . $table . " (vid, nid, $column) values (" . $node->vid . ',' . $node->nid . ",'". $node->from ."')";
	  db_query($sql);
  }
  
  
  //Add access rights
  if($rule['access_rights'] != null){
	  $grants = unserialize(base64_decode($rule['access_rights']));
	  if(is_array($grants)){
	    foreach ($grants as $grant) {
	      // Only write grants; denies are implicit.
	      if ($grant['grant_view'] || $grant['grant_update'] || $grant['grant_delete']) {
		db_query("INSERT INTO $node_access_table (nid, realm, gid, grant_view, grant_update, grant_delete) VALUES (%d, '%s', %d, %d, %d, %d)", $node->nid, $grant['realm'], $grant['gid'], $grant['grant_view'], $grant['grant_update'], $grant['grant_delete']);
	      }
	    }	  
	  }
  }
  
  // Clear cache so that anonymous users see all the pages and views.
  db_query("DELETE FROM ". $cache_table_page);
  if($cache_table_views != null) {
	  db_query("DELETE FROM ". $cache_table_views);
  }  
  return $node;  
}




  
?>
